#!/usr/bin/perl -w

#
# This script get movie data from www.estoescine.com website, 
# for spanish user.
#
# For more information on MythVideo's external movie look-up mechanism, see
# README file in this directory.
#
# Author: Alvaro Gonzalez Sola
# v1.0
#

use LWP::Simple;      # libwww-perl providing simple HTML get actions #esta libreria añade la funciones de get html
use HTML::Entities;
use URI::Escape;

eval "use DateTime::Format::Strptime"; my $has_date_format = $@ ? 0 : 1;

use vars qw($opt_h $opt_r $opt_d $opt_i $opt_v $opt_D $opt_M $opt_P);
use Getopt::Std; 

$title = "Estoescine Query"; 
$version = "v1.0";
$author = "Alvaro Gonzalez Sola";

binmode(STDOUT, ":utf8");

# display usage
sub usage {
   print "usage: $0 -hdrviMPD [parameters]\n";
   print "       -h           help\n";
   print "       -d           debug\n";
   print "       -r           dump raw query result data only\n";
   print "       -v           display version\n";
   print "       -i           display info\n";
   print "\n";
   print "       -M <query>    get movie list\n";
   print "       -P <movieid>  get movie poster\n";
   print "       -D <movieid>  get movie data\n";
   exit(-1);
}

# display 1-line of info that describes the version of the program
sub version {
   print "$title ($version) by $author\n"
} 

# display 1-line of info that can describe the type of query used
sub info {
   print "Performs queries using the www.estoescine.com website.\n";
}

# display detailed help 
sub help {
   version();
   info();
   usage();
}

sub trim {
   my ($str) = @_;
   $str =~ s/^\s+//;
   $str =~ s/\s+$//;
   return $str;
}

# returns text within 'data' between 'beg' and 'end' matching strings
sub parseBetween {
   my ($data, $beg, $end)=@_; # grab parameters

   my $ldata = lc($data);
   my $start = index($ldata, lc($beg)) + length($beg);
   my $finish = index($ldata, lc($end), $start);
   if ($start != (length($beg) -1) && $finish != -1) {
      my $result = substr($data, $start, $finish - $start);
      # return w/ decoded numeric character references
      # (see http://www.w3.org/TR/html4/charset.html#h-5.3.1)
      decode_entities($result);
      return $result;
   }
   return "";
}

# get Movie Data 
sub getMovieData {
   my ($movieid)=@_; # grab movieid parameter
   if (defined $opt_d) { printf("# looking for movie id: '%s'\n", $movieid);}

   my $name_link_pat = qr'<a href="/name/[^"]*">([^<]*)</a>'m;

   # get the search results  page
   my $request = "http://www.estoescine.com/films" . $movieid . ".htm";
   if (defined $opt_d) { printf("# request: '%s'\n", $request); }
   my $response = get $request;
   if (defined $opt_r) { printf("%s", $response); }

   # parse title and year
   my $year = "";
   my $title = parseBetween($response, "<title>", "</title>");
   if($title =~ m/(.+).*\(.*\).*- (\d+) - .*/)
   {
      $title = $1;
      $year = $2;
   }

   # parse director 
   my $data = parseBetween($response, "<b>Director</b>: ", "<br>");
   my $director = $data;

   # parse writer 
   $data = parseBetween($response, "<b>Gui","<br>");
   $data =~ m/.*: (.*)/ ;
   $data = $1;
   my $writer = $1;

   # change ", " for ","
   $writer =~ s/, /,/g;

   # parse release date
   my $releasedate = '';
   if ($has_date_format) {
      my $dtp = new DateTime::Format::Strptime(pattern => '%d %b %Y',
         on_error => 'undef');
      my $dt = $dtp->parse_datetime(parseBetween($response,
            "Estreno en españa</b>: ", "<br>"));
      if (defined($dt)) {
         $releasedate = $dt->strftime("%F");
      }
   }

   # parse plot, get it on "/sinopsis" page
   my $request_sinopsis = "http://www.estoescine.com/sinopsis" . $movieid . ".htm";
   if (defined $opt_d) { printf("%s", $request_sinopsis); }
   my $response_sinopsis = get $request_sinopsis;
   if (defined $opt_r) { printf("%s", $response_sinopsis); }
   my $plot = parseBetween($response_sinopsis, "<b>Sinopsis:</b></font>", "</p>");


   #www.estoescine.com don't have rating informatation,
   #in future release is possible connect to other website for get ratin information

   # parse user rating
   my $userrating;

   #www.estoescine.com don't have MPAA rating informatation,
   #in future release is possible connect to other website for get MPAA ratin information

   # parse MPAA rating
   my $movierating;

   # parse movie length
   my $rawruntime = trim(parseBetween($response, "n</b>: ", "<br>"));
   my $runtime = trim(parseBetween($rawruntime, "", " min"));

   # parse cast
   my $cast = "";
   $data = parseBetween($response, "b>Reparto</b>: ","<br>"); 
   $data =~ m/(.*)/;
   $cast = $1;
   $cast =~ s/, /,/g;

   # parse genres 
   my $lgenres = "";
   $lgenres = parseBetween($response, "nero</b>: ","</span>"); 

   # parse countries 
   $data = parseBetween($response, "s</b>: <img ","<br>"); 
   $data =~ m/ *(.*)\Z$/;
   $data = $1;
   my $lcountries = $data;

   # output fields (these field names must match what MythVideo is looking for)
   print "Title:$title\n";
   print "Year:$year\n";
   print "ReleaseDate:$releasedate\n";
   print "Director:$director\n";
   print "Plot:$plot\n";
   print "UserRating:$userrating\n";
   print "MovieRating:$movierating\n";
   print "Runtime:$runtime\n";
   print "Writers: $writer\n";
   print "Cast: $cast\n";
   print "Genres: $lgenres\n";
   print "Countries: $lcountries\n";
}

# dump Movie Poster
sub getMoviePoster {
   my ($movieid)=@_; # grab movieid parameter
   if (defined $opt_d) { printf("# looking for movie id: '%s'\n", $movieid);}

   # get the search results  page
   my $request = "http://www.estoescine.com/posterscine" . $movieid . ".htm";
   if (defined $opt_d) { printf("# request: '%s'\n", $request); }
   my $response = get $request;
   if (defined $opt_r) { printf("%s", $response); }

   if (!defined $response) {return;}

   my $uri = "";

   my $impsite = parseBetween($response, "<a href=", "<b>Cartel</b>: Espa");
   $imposite = parseBetween($imposite, "'","'");

   my $beg = "<br><span class=textocentralpresentacion>";
   my $end = " veces</span>";
   my $veces = 0;
   my $nveces = 0;

   my $data = parseBetween($response,"<a href='cartel.php","Anuncios");
   my $start = index($data, $beg);
   my $finish = index($data, $end, $start);

   #Extract all possible Poster, and get only spanish poster. Only select the most rating spanish poster.
   while ($start != -1 && $start < length($data)) {
      $start += length($beg);
      my $entry = substr($data, $start, $finish - $start);
      if($entry =~ m/Espa.a/){
        $entry =~ m/ (\d*)\Z/;
        $nveces = $1;
        if($veces < $nveces){
           $veces = $nveces;
           $entry =~ m/cartel.php\?(.*)' target=_blank/;
           $uri = $1;
        }
      }
      $start = index($data, $beg, $finish + 1);
      $finish = index($data, $end, $start);

      }

   # Make the poster url direction
   if(length($uri) > 0){
     $uri = ("http://www.estoescine.com/cartel.php?".$uri);
   }
   if (defined $opt_d) { printf("%s", $uri); }

   $response = get $uri;
   if (defined $opt_r) { printf("%s", $response); }

   # Extract image url from poster page
   $data = parseBetween($response,"http://imagenes.estoescine.com/"," style");
   $uri = ("http://imagenes.estoescine.com/".$data);
   print "$uri\n";
}

# dump Movie list:  1 entry per line, each line as 'movieid:Movie Title'
sub getMovieList {
   my ($filename)=@_; # grab parameters # adquiere los parametros

   # If we wanted to inspect the file for any reason we can do that now

   #
   # Convert filename into a query string 
   # (use same rules that Metadata::guesTitle does)
   my $query = $filename;
   $query = uri_unescape($query);  # in case it was escaped

   # Strip off the file extension
   if (rindex($query, '.') != -1) {
      $query = substr($query, 0, rindex($query, '.'));
   }

   # Strip off anything following '(' - people use this for general comments
   if (rindex($query, '(') != -1) {
      $query = substr($query, 0, rindex($query, '(')); 
   }

   # Strip off anything following '[' - people use this for general comments
   if (rindex($query, '[') != -1) {
      $query = substr($query, 0, rindex($query, '[')); 
   }

   # prepare the url
   $query = uri_escape($query);
   if (defined $opt_d) { 
      printf("# query: '%s'\n", $query);
   }

   # get the search results  page
   my $request="http://www.estoescine.com/buscador.php?letra=$query";
   if (defined $opt_d) { printf("# request: '%s'\n", $request); }

   my $response = "";
   $response = get $request;
   if (defined $opt_r) { print $response; }

   # parse movie list from matches
   my $beg = "<tr";
   my $end = "</tr>";
   my $count = 0;
   my @movies;

   my $data = parseBetween($response,"<tr valign=top><td height=15 align=\"left\"><img src=imagenes/pixel.gif width=1 height=1 border=0></td></tr>","<tr valign=top><td height=100% align=\"left\"><img src=imagenes/pixel.gif width=1 height=1 border=0></td></tr>");
   if ($data eq "") {
      if (defined $opt_d) { printf("# no results\n"); }
      return; 
   }

   my $start = index($data, $beg);
   my $finish = index($data, $end, $start);
   my $year;
   my $title;
   #Extract movie list, with id number, title and year
   while ($start != -1 && $start < length($data)) {
      $start += length($beg);
      my $entry = substr($data, $start, $finish - $start);
      $start = index($data, $beg, $finish + 1);
      $finish = index($data, $end, $start);

      my $title = "";
      my $year = "";
      my $movienum = "";

      if($entry =~ /<span class=textocentralpresentacion><a href='films(\d+).*<\/span><b>(.+)<\/b><\/a>.*\((\d+)\)/ ){
          $movienum = $1;
           $title = $2;
           $year = $3;
      } else {
           if (defined $opt_d) {
               print("Unrecognized entry format ($entry)\n");
           }
           next;
      }

      my $skip = 0;

      # add to array
      my $moviename = $title;
      if ($year ne "") {
         $moviename .= " ($year)";
      }
      $movies[$count++] = $movienum . ":" . $moviename;

   }

   # display array of values
   for $movie (@movies) { print "$movie\n"; }
}

#
# Main Program
#

# parse command line arguments 
getopts('ohrdivDMP');

# print out info 
if (defined $opt_v) { version(); exit 1; }
if (defined $opt_i) { info(); exit 1; }

# print out usage if needed
if (defined $opt_h || $#ARGV<0) { help(); }

# get Movie Data
if (defined $opt_D) {
   # take movieid from cmdline arg
   $movieid = shift || die "Usage : $0 -D <movieid>\n";
   getMovieData($movieid);
}

# get Poster
elsif (defined $opt_P) {
   # take movieid from cmdline arg
   $movieid = shift || die "Usage : $0 -P <movieid>\n";
   getMoviePoster($movieid);
}

# get Movie List
elsif (defined $opt_M) {
   # take query from cmdline arg
   $query = shift || die "Usage : $0 -M <query>\n";
   getMovieList($query);
}
